@@ -23,6 +23,9 @@
 #  define WIN32_NO_SOCKETS
 /* less I/O calls during each require */
 #  define PERL_DISABLE_PMC
+
+/* allow minitest to work */
+#  define PERL_TEXTMODE_SCRIPTS
 #endif
 
 #ifdef WIN32_NO_SOCKETS
@@ -57,9 +60,6 @@
 #  ifdef PERL_GLOBAL_STRUCT
 #    error PERL_GLOBAL_STRUCT cannot be defined with PERL_IMPLICIT_SYS
 #  endif
-#  define win32_get_privlib PerlEnv_lib_path
-#  define win32_get_sitelib PerlEnv_sitelib_path
-#  define win32_get_vendorlib PerlEnv_vendorlib_path
 #endif
 
 #ifdef __GNUC__
@@ -243,8 +243,6 @@ typedef long		gid_t;
 typedef unsigned short	mode_t;
 #endif
 
-#pragma  warning(disable: 4102)	/* "unreferenced label" */
-
 #if _MSC_VER < 1800
 #define isnan		_isnan	/* Defined already in VC++ 12.0 */
 #endif
@@ -259,6 +257,31 @@ typedef unsigned short	mode_t;
 #  pragma intrinsic(_rotl64,_rotr64)
 #endif
 
+#  pragma warning(push)
+#  pragma warning(disable:4756;disable:4056)
+PERL_STATIC_INLINE
+double S_Infinity() {
+    /* this is a real C literal which can get further constant folded
+       unlike using HUGE_VAL/_HUGE which are data symbol imports from the CRT
+       and therefore can not by folded by VC, an example of constant
+       folding INF is creating -INF */
+    return (DBL_MAX+DBL_MAX);
+}
+#  pragma warning(pop)
+#  define NV_INF S_Infinity()
+
+/* selectany allows duplicate and unused data symbols to be removed by
+   VC linker, if this were static, each translation unit will have its own,
+   usually unused __PL_nan_u, if this were plain extern it will cause link
+   to fail due to multiple definitions, since we dont know if we are being
+   compiled as static or DLL XS, selectany simply always works, the cost of
+   importing __PL_nan_u across DLL boundaries in size in the importing DLL
+   will be more than the 8 bytes it will take up being in each XS DLL if
+   that DLL actually uses __PL_nan_u */
+extern const __declspec(selectany) union { unsigned __int64 __q; double __d; }
+__PL_nan_u = { 0x7FF8000000000000UI64 };
+#  define NV_NAN ((NV)__PL_nan_u.__d)
+
 #endif /* _MSC_VER */
 
 #ifdef __MINGW32__		/* Minimal Gnu-Win32 */
@@ -326,8 +349,10 @@ extern  void	*sbrk(ptrdiff_t need);
 #endif
 extern	char *	getlogin(void);
 extern	int	chown(const char *p, uid_t o, gid_t g);
+#if !defined(__MINGW64_VERSION_MAJOR) || __MINGW64_VERSION_MAJOR < 4
 extern  int	mkstemp(const char *path);
 #endif
+#endif
 
 #undef	 Stat
 #define  Stat		win32_stat
@@ -366,6 +391,7 @@ typedef struct {
 
 DllExport void		win32_get_child_IO(child_IO_table* ptr);
 DllExport HWND		win32_create_message_window(void);
+DllExport int		win32_async_check(pTHX);
 
 extern int		my_fclose(FILE *);
 extern char *		win32_get_privlib(const char *pl, STRLEN *const len);
@@ -392,8 +418,8 @@ typedef  char *		caddr_t;	/* In malloc.c (core address). */
 
 #ifdef MYMALLOC
 #define EMBEDMYMALLOC	/**/
-/* #define USE_PERL_SBRK	/**/
-/* #define PERL_SBRK_VIA_MALLOC	/**/
+/* #define USE_PERL_SBRK	/ **/
+/* #define PERL_SBRK_VIA_MALLOC	/ **/
 #endif
 
 #ifdef PERL_TEXTMODE_SCRIPTS
@@ -470,8 +496,6 @@ struct interp_intern {
     Sighandler_t sigtable[SIG_SIZE];
 };
 
-DllExport int win32_async_check(pTHX);
-
 #define WIN32_POLL_INTERVAL 32768
 #define PERL_ASYNC_CHECK() if (w32_do_async || PL_sig_pending) win32_async_check(aTHX)
 
@@ -541,12 +565,12 @@ typedef struct {
     char pipech;    /* one char buffer for handles opened on pipes */
     int lockinitflag;
     CRITICAL_SECTION lock;
-/* this struct defintion breaks ABI compatibility with
+/* this struct definition breaks ABI compatibility with
  * not using, cl.exe's native VS version specitfic CRT. */
 #  if _MSC_VER >= 1400 && _MSC_VER < 1500
 #    error "This ioinfo struct is incomplete for Visual C 2005"
 #  endif
-/* VC 2005 CRT has atleast 3 different definitions of this struct based on the
+/* VC 2005 CRT has at least 3 different definitions of this struct based on the
  * CRT DLL's build number. */
 #  if _MSC_VER >= 1500
 #    ifndef _SAFECRT_IMPL
@@ -609,14 +633,6 @@ EXTERN_C _CRTIMP ioinfo* __pioinfo[];
 #endif
 #define PERLIO_NOT_STDIO 0
 
-#include "perlio.h"
-
-/*
- * This provides a layer of functions and macros to ensure extensions will
- * get to use the same RTL functions as the core.
- */
-#include "win32iop.h"
-
 #define EXEC_ARGV_CAST(x) ((const char *const *) x)
 
 DllExport void *win32_signal_context(void);
